"use strict";
exports.__esModule = true;
function getAddedSquaresLL(dlong, dlat) {
    return dlong * dlong + dlat * dlat;
}
function getSqDistLL(p1, p2) {
    return getAddedSquaresLL(p1.longitude - p2.longitude, p1.latitude - p2.latitude);
}
function getSqSegDistLL(p, p1, p2) {
    var long = p1.longitude, lat = p1.latitude, dlong = p2.longitude - long, dlat = p2.latitude - lat;
    if (dlong !== 0 || dlat !== 0) {
        var t = ((p.longitude - long) * dlong + (p.latitude - lat) * dlat) / getAddedSquaresLL(dlong, dlat);
        if (t > 1) {
            long = p2.longitude;
            lat = p2.latitude;
        }
        else if (t > 0) {
            long += dlong * t;
            lat += dlat * t;
        }
    }
    return getAddedSquaresLL(p.longitude - long, p.latitude - lat);
}
function simplifyRadialDistLL(points, sqTolerance) {
    var prevPoint = points[0], point;
    var newPoints = [prevPoint];
    for (var i = 1, len = points.length; i < len; i++) {
        point = points[i];
        if (getSqDistLL(point, prevPoint) > sqTolerance) {
            newPoints.push(point);
            prevPoint = point;
        }
    }
    if (prevPoint !== point) {
        newPoints.push(point);
    }
    return newPoints;
}
function simplifyDPStepLL(points, first, last, sqTolerance, simplified) {
    var maxSqDist = sqTolerance, index;
    for (var i = first + 1; i < last; i++) {
        var sqDist = getSqSegDistLL(points[i], points[first], points[last]);
        if (sqDist > maxSqDist) {
            index = i;
            maxSqDist = sqDist;
        }
    }
    if (maxSqDist > sqTolerance) {
        if (index - first > 1) {
            simplifyDPStepLL(points, first, index, sqTolerance, simplified);
        }
        simplified.push(points[index]);
        if (last - index > 1) {
            simplifyDPStepLL(points, index, last, sqTolerance, simplified);
        }
    }
}
function simplifyDouglasPeuckerLL(points, sqTolerance) {
    var last = points.length - 1;
    var simplified = [points[0]];
    simplifyDPStepLL(points, 0, last, sqTolerance, simplified);
    simplified.push(points[last]);
    return simplified;
}
function SimplifyLL(points, tolerance, highestQuality) {
    if (tolerance === void 0) { tolerance = 1; }
    if (highestQuality === void 0) { highestQuality = false; }
    if (points.length <= 2) {
        return points;
    }
    var sqTolerance = tolerance * tolerance;
    points = highestQuality ? points : simplifyRadialDistLL(points, sqTolerance);
    points = simplifyDouglasPeuckerLL(points, sqTolerance);
    return points;
}
exports.SimplifyLL = SimplifyLL;
